@@ -21,6 +21,9 @@ from gbp.git import GitRepository, GitRepositoryError
 from gbp.deb.pristinetar import DebianPristineTar
 from gbp.format import format_str
 
+import gbp.log
+
+
 class DebianGitRepository(GitRepository):
     """A git repository that holds the source of a Debian package"""
 
@@ -28,6 +31,30 @@ class DebianGitRepository(GitRepository):
         super(DebianGitRepository, self).__init__(path)
         self.pristine_tar = DebianPristineTar(self)
 
+    def tree_drop_dirs(self, tree, dirs):
+        """
+        Drop the given top level dirs from the given git tree
+        returning a new tree object.
+        """
+        objs = self.list_tree(tree)
+        new_tree_objs = []
+
+        for m, t, s, n in objs:
+            if not (n in dirs and t == 'tree'):
+                new_tree_objs.append((m, t, s, n))
+        new_tree = self.make_tree(new_tree_objs)
+        return new_tree
+
+    def tree_get_dir(self, tree, dir):
+        """
+        Get the SHA1 of directory in a given tree
+        """
+        toplevel = self.list_tree(tree)
+        for m, t, s, n in toplevel:
+            if n == dir and t == 'tree':
+                return s
+        return None
+
     def find_version(self, format, version):
         """
         Check if a certain version is stored in this repo and return the SHA1
@@ -48,7 +75,7 @@ class DebianGitRepository(GitRepository):
         """
         tag = self.version_to_tag(format, version)
         legacy_tag = self._build_legacy_tag(format, version)
-        if self.has_tag(tag): # new tags are injective
+        if self.has_tag(tag):  # new tags are injective
             # dereference to a commit object
             return self.rev_parse("%s^0" % tag)
         elif self.has_tag(legacy_tag):
@@ -59,7 +86,7 @@ class DebianGitRepository(GitRepository):
                 if line.endswith(" %s\n" % version):
                     # dereference to a commit object
                     return self.rev_parse("%s^0" % legacy_tag)
-                elif line.startswith('---'): # GPG signature start
+                elif line.startswith('---'):  # GPG signature start
                     return None
         return None
 
@@ -100,7 +127,7 @@ class DebianGitRepository(GitRepository):
         >>> DebianGitRepository._build_legacy_tag('upstream/%(version)s', '1:2.0~3')
         'upstream/2.0.3'
         """
-        if ':' in version: # strip of any epochs
+        if ':' in version:  # strip of any epochs
             version = version.split(':', 1)[1]
         version = version.replace('~', '.')
         return format % dict(version=version)
@@ -122,7 +149,7 @@ class DebianGitRepository(GitRepository):
 
         """
         return format_str(format, dict(version=DebianGitRepository._sanitize_version(version),
-                                       hversion=DebianGitRepository._sanitize_version(version).replace('.','-')))
+                                       hversion=DebianGitRepository._sanitize_version(version).replace('.', '-')))
 
     @staticmethod
     def _sanitize_version(version):
@@ -167,7 +194,7 @@ class DebianGitRepository(GitRepository):
 
     def has_pristine_tar_branch(self):
         """
-        Wheter the repo has a I{pristine-tar} branch.
+        Whether the repo has a I{pristine-tar} branch.
 
         @return: C{True} if the repo has pristine-tar commits already, C{False}
             otherwise
@@ -175,4 +202,25 @@ class DebianGitRepository(GitRepository):
         """
         return True if self.has_branch(self.pristine_tar_branch) else False
 
+    def create_pristinetar_commits(self, upstream_tree, tarball, component_tarballs):
+        """
+        Create pristine-tar commits for a package with main tarball tarball
+        and (optionl) component tarballs based on upstream_tree
+
+        @param tarball: path to main tarball
+        @param component_tarballs: C{list} of C{tuple}s of component
+            name and path to additional tarball
+        @param upstream_tree: the treeish in the git repo to create the commits against
+        """
+        components = [c for (c, t) in component_tarballs]
+        main_tree = self.tree_drop_dirs(upstream_tree, components)
+
+        for component, name in component_tarballs:
+            subtree = self.tree_get_dir(upstream_tree, component)
+            if not subtree:
+                raise GitRepositoryError("No tree for '%s' found in '%s' to create pristine tar commit from" % (component, upstream_tree))
+            gbp.log.debug("Creating pristine tar commit '%s' from '%s'" % (component, subtree))
+            self.pristine_tar.commit(name, subtree)
+        self.pristine_tar.commit(tarball, main_tree)
+
 # vim:et:ts=4:sw=4:et:sts=4:ai:set list listchars=tab\:»·,trail\:·:
