@@ -1,4 +1,3 @@
-
 # vim: set fileencoding=utf-8 :
 #
 # (C) 2006-2016 Guido Günther <agx@sigxcpu.org>
@@ -18,9 +17,8 @@
 #
 """Build a Debian package out of a Git repository"""
 
-from six.moves import configparser
 import errno
-import os, os.path
+import os
 import shutil
 import sys
 import time
@@ -42,17 +40,20 @@ from gbp.scripts.common.buildpackage import (index_name, wc_name,
                                              git_archive_submodules,
                                              git_archive_single, dump_tree,
                                              write_wc, drop_index)
+from gbp.scripts.common import ExitCodes
+from gbp.scripts.common.hook import Hook
 from gbp.pkg import compressor_opts, compressor_aliases, parse_archive_filename
 
-#{ upstream tarball preparation
-def git_archive(repo, cp, output_dir, treeish, comp_type, comp_level, with_submodules, subtarball=None):
+
+# upstream tarball preparation
+def git_archive(repo, cp, output_dir, treeish, comp_type, comp_level, with_submodules, component=None):
     "create a compressed orig tarball in output_dir using git_archive"
     try:
         comp_opts = compressor_opts[comp_type][0]
     except KeyError:
         raise GbpError("Unsupported compression type '%s'" % comp_type)
 
-    output = os.path.join(output_dir, du.orig_file(cp, comp_type, subtarball=subtarball))
+    output = os.path.join(output_dir, du.orig_file(cp, comp_type, component=component))
     prefix = "%s-%s" % (cp['Source'], cp['Upstream-Version'])
 
     try:
@@ -89,8 +90,8 @@ def prepare_upstream_tarball(repo, cp, options, tarball_dir, output_dir):
                                         options.tarball_dir)
 
     orig_files = [du.orig_file(cp, options.comp_type)]
-    if options.subtarballs:
-        orig_files += [du.orig_file(cp, options.comp_type, sub) for sub in options.subtarballs]
+    if options.components:
+        orig_files += [du.orig_file(cp, options.comp_type, c) for c in options.components]
 
     # look in tarball_dir first, if found force a symlink to it
     if options.tarball_dir:
@@ -123,9 +124,8 @@ def pristine_tar_commit(repo, cp, options, output_dir, orig_file, upstream_tree)
                       archive)
         repo.pristine_tar.commit(archive, upstream_tree)
 
-#}
 
-#{ Functions to handle export-dir
+# Functions to handle export-dir
 def write_tree(repo, options):
     """
     Write a tree of the index or working copy if necessary
@@ -207,10 +207,10 @@ def extract_orig(orig_tarball, dest_dir):
     if os.path.exists(underlay_debian_dir) and os.path.exists(format_file):
         format = DebianSourceFormat.parse_file(format_file)
         if format.version in ['2.0', '3.0']:
-            gbp.log.info("Removing debian/ from unpacked upstream source at %s" % underlay_debian_dir)
+            gbp.log.info("Removing debian/ from unpacked upstream "
+                         "source at %s" % underlay_debian_dir)
             shutil.rmtree(underlay_debian_dir)
 
-#}
 
 def source_vfs(repo, options, tree):
     """Init source package info either from git or from working copy"""
@@ -220,7 +220,7 @@ def source_vfs(repo, options, tree):
             source = DebianSource(GitVfs(repo, tree))
         else:
             source = DebianSource('.')
-            source.is_native() # check early if this works
+            source.is_native()  # check early if this works
     except Exception as e:
         raise GbpError("Can't determine package type: %s" % e)
     return source
@@ -253,26 +253,27 @@ def pristine_tar_build_orig(repo, cp, output_dir, options):
                          repo.pristine_tar.branch)
 
         # Make sure the upstream tree exists, it might not be there in
-        # case of subtarballs since we don't keep a ref to it
-        if options.subtarballs:
+        # case of components since we don't keep a ref to it
+        if options.components:
             try:
                 upstream_tag = repo.version_to_tag(options.upstream_tag, cp.upstream_version)
                 tree_name = "%s^{tree}" % upstream_tag
-                repo.tree_drop_dirs(tree_name, options.subtarballs)
+                repo.tree_drop_dirs(tree_name, options.components)
             except GitRepositoryError:
-                raise GbpError("Couldn't find upstream tree '%s' to create orig tarball via pristine-tar" % tree_name)
+                raise GbpError("Couldn't find upstream tree '%s' to create "
+                               "orig tarball via pristine-tar" % tree_name)
 
         try:
             repo.pristine_tar.checkout(cp.name,
                                        cp.upstream_version,
                                        options.comp_type,
                                        output_dir)
-            for subtarball in options.subtarballs:
+            for component in options.components:
                 repo.pristine_tar.checkout(cp.name,
                                            cp.upstream_version,
                                            options.comp_type,
                                            output_dir,
-                                           subtarball=subtarball)
+                                           component=component)
             return True
         except CommandExecFailed:
             if options.pristine_tar_commit:
@@ -321,26 +322,27 @@ def git_archive_build_orig(repo, cp, output_dir, options):
                                             upstream_tree))
     gbp.log.debug("Building upstream tarball with compression '%s -%s'" %
                   (options.comp_type, options.comp_level))
-    main_tree = repo.tree_drop_dirs(upstream_tree, options.subtarballs)
+    main_tree = repo.tree_drop_dirs(upstream_tree, options.components)
     if not git_archive(repo, cp, output_dir, main_tree,
                        options.comp_type,
                        options.comp_level,
                        options.with_submodules):
         raise GbpError("Cannot create upstream tarball at '%s'" % output_dir)
-    for subtarball in options.subtarballs:
-        subtree = repo.tree_get_dir(upstream_tree, subtarball)
+    for component in options.components:
+        subtree = repo.tree_get_dir(upstream_tree, component)
         if not subtree:
-            raise GbpError("No tree for '%s' found in '%s' to create subtarball from" % (subtarball, upstream_tree))
-        gbp.log.info("Creating subtarball '%s' from '%s'" % (du.orig_file(cp,
-                                                                          options.comp_type,
-                                                                          subtarball=subtarball),
-                                                             subtree))
+            raise GbpError("No tree for '%s' found in '%s' to create additional tarball from"
+                           % (component, upstream_tree))
+        gbp.log.info("Creating additional tarball '%s' from '%s'"
+                     % (du.orig_file(cp, options.comp_type, component=component),
+                        subtree))
         if not git_archive(repo, cp, output_dir, subtree,
                            options.comp_type,
                            options.comp_level,
                            options.with_submodules,
-                           subtarball=subtarball):
-            raise GbpError("Cannot create upstream component tarball %s at '%s'" % (subtarball, output_dir))
+                           component=component):
+            raise GbpError("Cannot create additional tarball %s at '%s'"
+                           % (component, output_dir))
     return upstream_tree
 
 
@@ -410,7 +412,7 @@ def get_pbuilder_dist(options, repo, native=False):
                 vendor = du.get_vendor().lower()
                 suite = parts[1]
                 if vendor == parts[0]:
-                    dist = '' if suite == 'sid' else suite
+                    dist = '' if suite in ['sid', 'master'] else suite
                 else:
                     dist = '%s_%s' % (parts[0], suite)
             elif len(parts) == 1 and native and branch in ['master', 'sid']:
@@ -487,32 +489,32 @@ def changes_file_suffix(dpkg_args):
         return os.getenv('ARCH', None) or du.get_arch()
 
 
-def md(a, b):
-    "Merge two dictionaires a and b"
-    c = a.copy()
-    c.update(b)
-    return c
-
-
 def build_parser(name, prefix=None):
     try:
         parser = GbpOptionParserDebian(command=os.path.basename(name), prefix=prefix)
-    except configparser.ParsingError as err:
+    except GbpError as err:
         gbp.log.err(err)
         return None
 
-    tag_group = GbpOptionGroup(parser, "tag options", "options related to git tag creation")
-    branch_group = GbpOptionGroup(parser, "branch options", "branch layout options")
-    cmd_group = GbpOptionGroup(parser, "external command options", "how and when to invoke external commands and hooks")
-    orig_group = GbpOptionGroup(parser, "orig tarball options", "options related to the creation of the orig tarball")
-    export_group = GbpOptionGroup(parser, "export build-tree options", "alternative build tree related options")
-    parser.add_option_group(tag_group)
-    parser.add_option_group(orig_group)
-    parser.add_option_group(branch_group)
-    parser.add_option_group(cmd_group)
-    parser.add_option_group(export_group)
+    tag_group = GbpOptionGroup(parser,
+                               "tag options",
+                               "options related to git tag creation")
+    orig_group = GbpOptionGroup(parser,
+                                "orig tarball options",
+                                "options related to the creation of the orig tarball")
+    branch_group = GbpOptionGroup(parser,
+                                  "branch options",
+                                  "branch layout options")
+    cmd_group = GbpOptionGroup(parser,
+                               "external command options",
+                               "how and when to invoke external commands and hooks")
+    export_group = GbpOptionGroup(parser,
+                                  "export build-tree options",
+                                  "alternative build tree related options")
+    for group in [tag_group, orig_group, branch_group, cmd_group, export_group]:
+        parser.add_option_group(group)
 
-    parser.add_boolean_config_file_option(option_name = "ignore-new", dest="ignore_new")
+    parser.add_boolean_config_file_option(option_name="ignore-new", dest="ignore_new")
     parser.add_option("--git-verbose", action="store_true", dest="verbose", default=False,
                       help="verbose command execution")
     parser.add_config_file_option(option_name="color", dest="color", type='tristate')
@@ -520,11 +522,11 @@ def build_parser(name, prefix=None):
                                   dest="color_scheme")
     parser.add_config_file_option(option_name="notify", dest="notify", type='tristate')
     tag_group.add_option("--git-tag", action="store_true", dest="tag", default=False,
-                      help="create a tag after a successful build")
+                         help="create a tag after a successful build")
     tag_group.add_option("--git-tag-only", action="store_true", dest="tag_only", default=False,
-                      help="don't build, only tag and run the posttag hook")
+                         help="don't build, only tag and run the posttag hook")
     tag_group.add_option("--git-retag", action="store_true", dest="retag", default=False,
-                      help="don't fail if the tag already exists")
+                         help="don't fail if the tag already exists")
     tag_group.add_boolean_config_file_option(option_name="sign-tags", dest="sign_tags")
     tag_group.add_config_file_option(option_name="keyid", dest="keyid")
     tag_group.add_config_file_option(option_name="debian-tag", dest="debian_tag")
@@ -535,44 +537,53 @@ def build_parser(name, prefix=None):
     orig_group.add_boolean_config_file_option(option_name="pristine-tar-commit",
                                               dest="pristine_tar_commit")
     orig_group.add_config_file_option(option_name="force-create", dest="force_create",
-                      help="force creation of orig tarball", action="store_true")
+                                      help="force creation of orig tarball", action="store_true")
     orig_group.add_config_file_option(option_name="no-create-orig", dest="no_create_orig",
-                      help="don't create orig tarball", action="store_true")
+                                      help="don't create orig tarball", action="store_true")
     orig_group.add_config_file_option(option_name="tarball-dir", dest="tarball_dir", type="path",
-                      help="location to look for external tarballs")
+                                      help="location to look for external tarballs")
     orig_group.add_config_file_option(option_name="compression", dest="comp_type",
-                      help="Compression type, default is '%(compression)s'")
+                                      help="Compression type, default is '%(compression)s'")
     orig_group.add_config_file_option(option_name="compression-level", dest="comp_level",
-                      help="Compression level, default is '%(compression-level)s'")
-    orig_group.add_option("--git-subtarball", action="append", metavar='SUBTARBALL',
-                          dest="subtarballs", help="subtarsball to generate, can be given multiple times", default=[])
+                                      help="Compression level, default is '%(compression-level)s'")
+    orig_group.add_config_file_option("component", action="append", metavar='COMPONENT',
+                                      dest="components")
     branch_group.add_config_file_option(option_name="upstream-branch", dest="upstream_branch")
     branch_group.add_config_file_option(option_name="debian-branch", dest="debian_branch")
-    branch_group.add_boolean_config_file_option(option_name = "ignore-branch", dest="ignore_branch")
-    branch_group.add_boolean_config_file_option(option_name = "submodules", dest="with_submodules")
+    branch_group.add_boolean_config_file_option(option_name="ignore-branch", dest="ignore_branch")
+    branch_group.add_boolean_config_file_option(option_name="submodules", dest="with_submodules")
     cmd_group.add_config_file_option(option_name="builder", dest="builder",
-                      help="command to build the Debian package, default is '%(builder)s'")
+                                     help="command to build the Debian package, "
+                                          "default is '%(builder)s'")
     cmd_group.add_config_file_option(option_name="cleaner", dest="cleaner",
-                      help="command to clean the working copy, default is '%(cleaner)s'")
+                                     help="command to clean the working copy, "
+                                          "default is '%(cleaner)s'")
     cmd_group.add_config_file_option(option_name="prebuild", dest="prebuild",
-                      help="hook to run before a build, default is '%(prebuild)s'")
+                                     help="hook to run before a build, "
+                                          "default is '%(prebuild)s'")
     cmd_group.add_config_file_option(option_name="postexport", dest="postexport",
-                      help="hook to run after exporting the source tree, default is '%(postexport)s'")
+                                     help="hook to run after exporting the source tree, "
+                                          "default is '%(postexport)s'")
     cmd_group.add_config_file_option(option_name="postbuild", dest="postbuild",
-                      help="hook run after a successful build, default is '%(postbuild)s'")
+                                     help="hook run after a successful build, "
+                                          "default is '%(postbuild)s'")
     cmd_group.add_config_file_option(option_name="posttag", dest="posttag",
-                      help="hook run after a successful tag operation, default is '%(posttag)s'")
+                                     help="hook run after a successful tag operation, "
+                                          "default is '%(posttag)s'")
     cmd_group.add_boolean_config_file_option(option_name="pbuilder", dest="use_pbuilder")
     cmd_group.add_boolean_config_file_option(option_name="qemubuilder", dest="use_qemubuilder")
     cmd_group.add_config_file_option(option_name="dist", dest="pbuilder_dist")
     cmd_group.add_config_file_option(option_name="arch", dest="pbuilder_arch")
-    cmd_group.add_boolean_config_file_option(option_name = "pbuilder-autoconf", dest="pbuilder_autoconf")
+    cmd_group.add_boolean_config_file_option(option_name="pbuilder-autoconf",
+                                             dest="pbuilder_autoconf")
     cmd_group.add_config_file_option(option_name="pbuilder-options", dest="pbuilder_options")
     cmd_group.add_boolean_config_file_option(option_name="hooks", dest="hooks")
     export_group.add_config_file_option(option_name="export-dir", dest="export_dir", type="path",
-                      help="before building the package export the source into EXPORT_DIR, default is '%(export-dir)s'")
+                                        help="before building the package export the source into EXPORT_DIR, "
+                                             "default is '%(export-dir)s'")
     export_group.add_config_file_option("export", dest="export",
-                      help="export treeish object TREEISH, default is '%(export)s'", metavar="TREEISH")
+                                        help="export treeish object TREEISH, "
+                                             "default is '%(export)s'", metavar="TREEISH")
     export_group.add_boolean_config_file_option(option_name="purge", dest="purge")
     export_group.add_option("--git-dont-purge", action="store_true", dest="dont_purge", default=False,
                             help="deprecated, use --git-no-purge instead")
@@ -581,11 +592,11 @@ def build_parser(name, prefix=None):
 
 
 def parse_args(argv, prefix):
-    args = [ arg for arg in argv[1:] if arg.find('--%s' % prefix) == 0 ]
-    dpkg_args = [ arg for arg in argv[1:] if arg.find('--%s' % prefix) == -1 ]
+    args = [arg for arg in argv[1:] if arg.find('--%s' % prefix) == 0]
+    dpkg_args = [arg for arg in argv[1:] if arg.find('--%s' % prefix) == -1]
 
     # We handle these although they don't have a --git- prefix
-    for arg in [ "--help", "-h", "--version" ]:
+    for arg in ["--help", "-h", "--version"]:
         if arg in dpkg_args:
             args.append(arg)
 
@@ -599,7 +610,8 @@ def parse_args(argv, prefix):
         disable_hooks(options)
     if options.retag:
         if not options.tag and not options.tag_only:
-            gbp.log.err("'--%sretag' needs either '--%stag' or '--%stag-only'" % (prefix, prefix, prefix))
+            gbp.log.err("'--%sretag' needs either '--%stag' or '--%stag-only'"
+                        % (prefix, prefix, prefix))
             return None, None, None
 
     if options.overlay and not options.export_dir:
@@ -611,22 +623,13 @@ def parse_args(argv, prefix):
         gbp.log.warning("--git-dont-purge is depreceted, use --git-no-purge instead")
         options.purge = False
 
-    if options.subtarballs and options.pristine_tar_commit:
-        gbp.log.warning("Subtarballs specified, pristine-tar-commit not yet supported - disabling it.")
+    if options.components and options.pristine_tar_commit:
+        gbp.log.warning("Components specified, pristine-tar-commit not yet supported - disabling it.")
         options.pristine_tar_commit = False
 
     return options, args, dpkg_args
 
 
-class Hook(RunAtCommand):
-    "A hook run during the build"
-    def __init__(self, name, *args, **kwargs):
-        if 'shell' not in kwargs:
-            kwargs['shell'] = True
-        RunAtCommand.__init__(self, *args, **kwargs)
-        self.run_error = '%s-hook %s' % (name, self.run_error)
-
-
 def main(argv):
     retval = 0
     prefix = "git-"
@@ -637,15 +640,13 @@ def main(argv):
     options, gbp_args, dpkg_args = parse_args(argv, prefix)
 
     if not options:
-        return 1
+        return ExitCodes.parse_error
 
     try:
         repo = DebianGitRepository(os.path.curdir)
     except GitRepositoryError:
         gbp.log.err("%s is not a git repository" % (os.path.abspath('.')))
         return 1
-    else:
-        repo_dir = os.path.abspath(os.path.curdir)
 
     try:
         Command(options.cleaner, shell=True)()
@@ -700,9 +701,9 @@ def main(argv):
                 # Run postexport hook
                 if options.postexport:
                     Hook('Postexport', options.postexport,
-                         extra_env=md(hook_env,
-                                      {'GBP_GIT_DIR': repo.git_dir,
-                                       'GBP_TMP_DIR': tmp_dir})
+                         extra_env=Hook.md(hook_env,
+                                           {'GBP_GIT_DIR': repo.git_dir,
+                                            'GBP_TMP_DIR': tmp_dir})
                          )(dir=tmp_dir)
 
                 major = (source.changelog.debian_version if source.is_native()
@@ -716,23 +717,21 @@ def main(argv):
                 if not source.is_native() and options.postexport:
                     prepare_upstream_tarball(repo, source.changelog, options, tarball_dir,
                                              output_dir)
-
-            if options.export_dir:
                 build_dir = export_dir
             else:
-                build_dir = repo_dir
+                build_dir = repo.path
 
             if options.prebuild:
                 Hook('Prebuild', options.prebuild,
-                     extra_env=md(hook_env,
-                                  {'GBP_GIT_DIR': repo.git_dir,
-                                   'GBP_BUILD_DIR': build_dir})
+                     extra_env=Hook.md(hook_env,
+                                       {'GBP_GIT_DIR': repo.git_dir,
+                                        'GBP_BUILD_DIR': build_dir})
                      )(dir=build_dir)
 
             # Finally build the package:
             RunAtCommand(options.builder, dpkg_args, shell=True,
-                         extra_env=md(build_env,
-                                      {'GBP_BUILD_DIR': build_dir})
+                         extra_env=Hook.md(build_env,
+                                           {'GBP_BUILD_DIR': build_dir})
                          )(dir=build_dir)
             if options.postbuild:
                 changes = os.path.abspath("%s/../%s_%s_%s.changes" %
@@ -742,9 +741,9 @@ def main(argv):
                                            changes_file_suffix(dpkg_args)))
                 gbp.log.debug("Looking for changes file %s" % changes)
                 Hook('Postbuild', options.postbuild,
-                     extra_env=md(hook_env,
-                                  {'GBP_CHANGES_FILE': changes,
-                                   'GBP_BUILD_DIR': build_dir})
+                     extra_env=Hook.md(hook_env,
+                                       {'GBP_CHANGES_FILE': changes,
+                                        'GBP_BUILD_DIR': build_dir})
                      )()
         if options.tag or options.tag_only:
             tag = repo.version_to_tag(options.debian_tag, source.changelog.version)
@@ -762,11 +761,14 @@ def main(argv):
             if options.posttag:
                 sha = repo.rev_parse("%s^{}" % tag)
                 Hook('Posttag', options.posttag,
-                     extra_env=md(hook_env,
-                                  {'GBP_TAG': tag,
-                                   'GBP_BRANCH': branch or '(no branch)',
-                                   'GBP_SHA1': sha})
+                     extra_env=Hook.md(hook_env,
+                                       {'GBP_TAG': tag,
+                                        'GBP_BRANCH': branch or '(no branch)',
+                                        'GBP_SHA1': sha})
                      )()
+    except KeyboardInterrupt:
+        retval = 1
+        gbp.log.err("Interrupted. Aborting.")
     except CommandExecFailed:
         retval = 1
     except (GbpError, GitRepositoryError) as err:
