@@ -17,11 +17,12 @@
 #
 """Query and display config file values"""
 
-from six.moves import configparser
 import sys
-import os, os.path
+import os
 from gbp.config import GbpOptionParser
+from gbp.errors import GbpError
 from gbp.scripts.supercommand import import_command
+from gbp.scripts.common import ExitCodes
 import gbp.log
 
 
@@ -29,7 +30,7 @@ def build_parser(name):
     try:
         parser = GbpOptionParser(command=os.path.basename(name), prefix='',
                                  usage='%prog [options] command[.optionname] - display configuration settings')
-    except configparser.ParsingError as err:
+    except GbpError as err:
         gbp.log.err(err)
         return None
 
@@ -117,6 +118,10 @@ def main(argv):
     retval = 1
 
     (options, args) = parse_args(argv)
+
+    if options is None:
+        return ExitCodes.parse_error
+
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
 
     if not args:
