@@ -20,15 +20,14 @@
 
 from __future__ import print_function
 
-from six.moves import configparser
 import sys
-import os, os.path
+import os
 from six.moves import urllib
 import subprocess
-import tty, termios
+import tty
+import termios
 import re
-
-import six
+from six.moves import configparser
 
 from gbp.deb.changelog import ChangeLog, NoChangeLogError
 from gbp.command_wrappers import (CommandExecFailed, GitCommand)
@@ -36,8 +35,11 @@ from gbp.config import (GbpOptionParserDebian, GbpOptionGroup)
 from gbp.errors import GbpError
 from gbp.git import GitRepositoryError
 from gbp.deb.git import DebianGitRepository
+from gbp.scripts.common import ExitCodes
+
 import gbp.log
 
+
 def print_config(remote, branches):
     """
     Print out the git config to push to the newly created repo.
@@ -68,45 +70,11 @@ def print_config(remote, branches):
         remote = %s
         merge = refs/heads/%s""" % (branch, remote['name'], branch))
 
-def sort_dict(d):
-    """Return a sorted list of (key, value) tuples"""
-    s = []
-    for key in sorted(six.iterkeys(d)):
-        s.append((key, d[key]))
-    return s
 
-def parse_url(remote_url, name, pkg, template_dir=None):
+def parse_url(remote_url, name, pkg, template_dir=None, bare=True):
     """
     Sanity check our remote URL
 
-    >>> sort_dict(parse_url("ssh://host/path/%(pkg)s", "origin", "package"))
-    [('base', ''), ('dir', '/path/package'), ('host', 'host'), ('name', 'origin'), ('pkg', 'package'), ('port', None), ('scheme', 'ssh'), ('template-dir', None), ('url', 'ssh://host/path/package')]
-
-    >>> sort_dict(parse_url("ssh://host:22/path/repo.git", "origin", "package"))
-    [('base', ''), ('dir', '/path/repo.git'), ('host', 'host'), ('name', 'origin'), ('pkg', 'package'), ('port', '22'), ('scheme', 'ssh'), ('template-dir', None), ('url', 'ssh://host:22/path/repo.git')]
-
-    >>> sort_dict(parse_url("ssh://host:22/~/path/%(pkg)s.git", "origin", "package"))
-    [('base', '~/'), ('dir', 'path/package.git'), ('host', 'host'), ('name', 'origin'), ('pkg', 'package'), ('port', '22'), ('scheme', 'ssh'), ('template-dir', None), ('url', 'ssh://host:22/~/path/package.git')]
-
-    >>> sort_dict(parse_url("ssh://host:22/~user/path/%(pkg)s.git", "origin", "package", "/doesnot/exist"))
-    [('base', '~user/'), ('dir', 'path/package.git'), ('host', 'host'), ('name', 'origin'), ('pkg', 'package'), ('port', '22'), ('scheme', 'ssh'), ('template-dir', '/doesnot/exist'), ('url', 'ssh://host:22/~user/path/package.git')]
-
-    >>> parse_url("git://host/repo.git", "origin", "package")
-    Traceback (most recent call last):
-        ...
-    GbpError: URL must use ssh protocol.
-    >>> parse_url("ssh://host/path/repo", "origin", "package")
-    Traceback (most recent call last):
-        ...
-    GbpError: URL needs to contain either a repository name or '%(pkg)s'
-    >>> parse_url("ssh://host:asdf/path/%(pkg)s.git", "origin", "package")
-    Traceback (most recent call last):
-        ...
-    GbpError: URL contains invalid port.
-    >>> parse_url("ssh://host/~us er/path/%(pkg)s.git", "origin", "package")
-    Traceback (most recent call last):
-        ...
-    GbpError: URL contains invalid ~username expansion.
     """
     frags = urllib.parse.urlparse(remote_url)
     if frags.scheme in ['ssh', 'git+ssh', '']:
@@ -114,7 +82,7 @@ def parse_url(remote_url, name, pkg, template_dir=None):
     else:
         raise GbpError("URL must use ssh protocol.")
 
-    if not '%(pkg)s' in remote_url and not remote_url.endswith(".git"):
+    if '%(pkg)s' not in remote_url and not remote_url.endswith(".git"):
         raise GbpError("URL needs to contain either a repository name or '%(pkg)s'")
 
     if ":" in frags.netloc:
@@ -135,45 +103,50 @@ def parse_url(remote_url, name, pkg, template_dir=None):
         base = ""
         path = frags.path
 
-    remote = { 'pkg' : pkg,
-               'url' : remote_url % { 'pkg': pkg },
-               'dir' : path % { 'pkg': pkg },
-               'base': base,
-               'host': host,
-               'port': port,
-               'name': name,
-               'scheme': scheme,
-               'template-dir': template_dir}
+    remote = {'pkg': pkg,
+              'url': remote_url % {'pkg': pkg},
+              'dir': path % {'pkg': pkg},
+              'base': base,
+              'host': host,
+              'port': port,
+              'name': name,
+              'scheme': scheme,
+              'template-dir': template_dir,
+              'bare': bare}
     return remote
 
 
 def build_remote_script(remote, branch):
     """
     Create the script that will be run on the remote side
-    >>> build_remote_script({'base': 'base', 'dir': 'dir', 'pkg': 'pkg', 'template-dir': None}, 'branch')
-    '\\nset -e\\numask 002\\nif [ -d base"dir" ]; then\\n  echo "Repository at "basedir" already exists - giving up."\\n  exit 1\\nfi\\nmkdir -p base"dir"\\ncd base"dir"\\ngit init --bare --shared\\necho "pkg packaging" > description\\necho "ref: refs/heads/branch" > HEAD\\n'
-    >>> build_remote_script({'base': 'base', 'dir': 'dir', 'pkg': 'pkg', 'template-dir': '/doesnot/exist'}, 'branch')
-    '\\nset -e\\numask 002\\nif [ -d base"dir" ]; then\\n  echo "Repository at "basedir" already exists - giving up."\\n  exit 1\\nfi\\nmkdir -p base"dir"\\ncd base"dir"\\ngit init --bare --shared --template=/doesnot/exist\\necho "pkg packaging" > description\\necho "ref: refs/heads/branch" > HEAD\\n'
     """
     args = remote
     args['branch'] = branch
-    args['git-init-args'] = '--bare --shared'
+    args['git-init-args'] = '--shared'
+    if args['bare']:
+        args['git-init-args'] += ' --bare'
+        args['checkout_cmd'] = ''
+        args['git_dir'] = '.'
+    else:
+        args['checkout_cmd'] = 'git checkout -f'
+        args['git_dir'] = '.git'
     if args['template-dir']:
         args['git-init-args'] += (' --template=%s'
-                                    % args['template-dir'])
-    remote_script_pattern = ['',
-      'set -e',
-      'umask 002',
-      'if [ -d %(base)s"%(dir)s" ]; then',
-      '  echo "Repository at \"%(base)s%(dir)s\" already exists - giving up."',
-      '  exit 1',
-      'fi',
-      'mkdir -p %(base)s"%(dir)s"',
-      'cd %(base)s"%(dir)s"',
-      'git init %(git-init-args)s',
-      'echo "%(pkg)s packaging" > description',
-      'echo "ref: refs/heads/%(branch)s" > HEAD',
-      '' ]
+                                  % args['template-dir'])
+    remote_script_pattern = \
+        ['',
+         'set -e',
+         'umask 002',
+         'if [ -d %(base)s"%(dir)s" ]; then',
+         '  echo "Repository at \"%(base)s%(dir)s\" already exists - giving up."',
+         '  exit 1',
+         'fi',
+         'mkdir -p %(base)s"%(dir)s"',
+         'cd %(base)s"%(dir)s"',
+         'git init %(git-init-args)s',
+         'echo "%(pkg)s packaging" > %(git_dir)s/description',
+         'echo "ref: refs/heads/%(branch)s" > %(git_dir)s/HEAD',
+         '']
     remote_script = '\n'.join(remote_script_pattern) % args
     return remote_script
 
@@ -212,7 +185,7 @@ def read_yn():
         if old_settings:
             termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
 
-    if ch in ( 'y', 'Y' ):
+    if ch in ('y', 'Y'):
         return True
     else:
         return False
@@ -237,7 +210,7 @@ def build_parser(name, sections=[]):
                                        usage='%prog [options] - '
                                        'create a remote repository',
                                        sections=sections)
-    except configparser.ParsingError as err:
+    except (GbpError, configparser.NoSectionError) as err:
         gbp.log.err(err)
         return None
 
@@ -255,6 +228,8 @@ def build_parser(name, sections=[]):
                                                 dest="pristine_tar")
     branch_group.add_boolean_config_file_option(option_name="track",
                                                 dest='track')
+    branch_group.add_boolean_config_file_option(option_name="bare",
+                                                dest='bare')
     parser.add_option("-v", "--verbose",
                       action="store_true",
                       dest="verbose",
@@ -291,7 +266,7 @@ def parse_args(argv, sections=[]):
     # section to parse, this makes e.g. --help work as expected:
     for arg in argv:
         if arg.startswith('--remote-config='):
-            sections = ['remote-config %s' % arg.split('=',1)[1]]
+            sections = ['remote-config %s' % arg.split('=', 1)[1]]
             break
     else:
         sections = []
@@ -308,11 +283,10 @@ def main(argv):
     changelog = 'debian/changelog'
     cmd = []
 
-    try:
-        options, args = parse_args(argv)
-    except Exception as e:
-        print("%s" % e, file=sys.stderr)
-        return 1
+    options, args = parse_args(argv)
+
+    if not options:
+        return ExitCodes.parse_error
 
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
     try:
@@ -324,12 +298,12 @@ def main(argv):
     try:
         branches = []
 
-        for branch in [ options.debian_branch, options.upstream_branch ]:
+        for branch in [options.debian_branch, options.upstream_branch]:
             if repo.has_branch(branch):
-                branches += [ branch ]
+                branches += [branch]
 
         if repo.has_pristine_tar_branch() and options.pristine_tar:
-            branches += [ repo.pristine_tar_branch ]
+            branches += [repo.pristine_tar_branch]
 
         try:
             cp = ChangeLog(filename=changelog)
@@ -345,7 +319,8 @@ def main(argv):
         remote = parse_url(options.remote_url,
                            options.name,
                            pkg,
-                           options.template_dir)
+                           options.template_dir,
+                           options.bare)
         if repo.has_remote_repo(options.name):
             raise GbpError("You already have a remote name '%s' defined for this repository." % options.name)
 
@@ -374,8 +349,10 @@ def main(argv):
         else:
             gbp.log.info("You can now add:")
             print_config(remote, branches)
-            gbp.log.info("to your .git/config to 'gbp-pull' and 'git push' in the future.")
-
+            gbp.log.info("to your .git/config to 'gbp pull' and 'git push' in the future.")
+    except KeyboardInterrupt:
+        retval = 1
+        gbp.log.err("Interrupted. Aborting.")
     except CommandExecFailed:
         retval = 1
     except (GbpError, GitRepositoryError) as err:
