@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 #
-# (C) 2006, 2007, 2009, 2011, 2015 Guido Guenther <agx@sigxcpu.org>
+# (C) 2006, 2007, 2009, 2011, 2015, 2016 Guido Guenther <agx@sigxcpu.org>
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation; either version 2 of the License, or
@@ -17,23 +17,154 @@
 #
 """Import a new upstream version into a Git repository"""
 
-from six.moves import configparser
 import os
 import sys
 import tempfile
 import gbp.command_wrappers as gbpc
 from gbp.deb import (DebianPkgPolicy, parse_changelog_repo)
-from gbp.deb.upstreamsource import DebianUpstreamSource
+from gbp.deb.upstreamsource import DebianUpstreamSource, unpack_component_tarball
 from gbp.deb.uscan import (Uscan, UscanError)
 from gbp.deb.changelog import ChangeLog, NoChangeLogError
 from gbp.deb.git import (GitRepositoryError, DebianGitRepository)
 from gbp.config import GbpOptionParserDebian, GbpOptionGroup, no_upstream_branch_msg
 from gbp.errors import GbpError
+from gbp.pkg import parse_archive_filename
 from gbp.format import format_str
 import gbp.log
+from gbp.scripts.common import ExitCodes
 from gbp.scripts.common.import_orig import (orig_needs_repack, cleanup_tmp_tree,
                                             ask_package_name, ask_package_version,
                                             repack_source, is_link_target, download_orig)
+from gbp.scripts.common.hook import Hook
+
+
+class RollbackError(GitRepositoryError):
+    """
+    An error raised if the actual rollback failed
+    """
+    def __init__(self, errors):
+        self.msg = "Automatic rollback failed"
+        super(RollbackError, self).__init__(self.msg)
+        self.errors = errors
+
+    def __str__(self):
+        return "%s %s" % (self.msg, self.errors)
+
+
+class ImportOrigDebianGitRepository(DebianGitRepository):
+    """
+    Like a DebianGitRepository but can also perform rollbacks and knows
+    about some of the inner workings upstream vcs_tag, …
+    """
+    def __init__(self, *args, **kwargs):
+        self.rollbacks = []
+        self.rollback_errors = []
+        DebianGitRepository.__init__(self, *args, **kwargs)
+
+    def has_rollbacks(self):
+        return len(self.rollbacks) > 0
+
+    def vcs_tag_parent(self, vcs_tag_format, version):
+        """If linking to the upstream VCS get the commit id"""
+        if vcs_tag_format:
+            return [self.rev_parse("%s^{}" % self.version_to_tag(vcs_tag_format, version))],
+        else:
+            return None
+
+    def rrr(self, refname, action, reftype):
+        """
+        Remember ref for rollback
+
+        @param refname: ref to roll back
+        @param action: the rollback action (delete, reset, ...)
+        @param reftype: the reference type (tag, branch, ...)
+        """
+        sha = None
+
+        if action == 'reset':
+            try:
+                sha = self.rev_parse(refname)
+            except GitRepositoryError as err:
+                gbp.log.warning("Failed to rev-parse %s: %s" % (refname, err))
+        elif action == 'delete':
+            pass
+        elif action == 'abortmerge':
+            pass
+        else:
+            raise GbpError("Unknown action %s for %s %s" % (action, reftype, refname))
+        self.rollbacks.append((refname, reftype, action, sha))
+
+    def rrr_branch(self, branchname, action='reset-or-delete'):
+        if action == 'reset-or-delete':
+            if self.has_branch(branchname):
+                return self.rrr(branchname, 'reset', 'branch')
+            else:
+                return self.rrr(branchname, 'delete', 'branch')
+        else:
+            return self.rrr(branchname, action, 'branch')
+
+    def rrr_tag(self, tagname, action='delete'):
+        return self.rrr(tagname, action, 'tag')
+
+    def rrr_merge(self, commit, action='abortmerge'):
+        return self.rrr(commit, action, 'commit')
+
+    def rollback(self):
+        """
+        Perform a complete rollback
+
+        Try to roll back as much as possible and remember what failed.
+        """
+        for (name, reftype, action, sha) in self.rollbacks:
+            try:
+                if action == 'delete':
+                    gbp.log.info('Rolling back %s %s by deleting it' % (reftype, name))
+                    if reftype == 'tag':
+                        self.delete_tag(name)
+                    elif reftype == 'branch':
+                        self.delete_branch(name)
+                    else:
+                        raise GitRepositoryError("Don't know how to delete %s %s" % (reftype, name))
+                elif action == 'reset' and reftype == 'branch':
+                    gbp.log.info('Rolling back branch %s by resetting it to %s' % (name, sha))
+                    self.update_ref("refs/heads/%s" % name, sha, msg="gbp import-orig: failure rollback of %s" % name)
+                elif action == 'abortmerge':
+                    gbp.log.info('Rolling back failed merge of %s' % name)
+                    self.abort_merge()
+                else:
+                    raise GitRepositoryError("Don't know how to %s %s %s" % (action, reftype, name))
+            except GitRepositoryError as e:
+                self.rollback_errors.append((name, reftype, action, sha, e))
+        if self.rollback_errors:
+            raise RollbackError(self.rollback_errors)
+
+    # Wrapped methods for rollbacks
+    def create_tag(self, *args, **kwargs):
+        name = kwargs['name']
+        ret = super(ImportOrigDebianGitRepository, self).create_tag(*args, **kwargs)
+        self.rrr_tag(name)
+        return ret
+
+    def commit_dir(self, *args, **kwargs):
+        import_branch = kwargs['branch']
+        self.rrr_branch(import_branch)
+        return super(ImportOrigDebianGitRepository, self).commit_dir(*args, **kwargs)
+
+    def create_branch(self, *args, **kwargs):
+        branch = kwargs['branch']
+        ret = super(ImportOrigDebianGitRepository, self).create_branch(*args, **kwargs)
+        self.rrr_branch(branch, 'delete')
+        return ret
+
+    def merge(self, *args, **kwargs):
+        commit = args[0] if args else kwargs['commit']
+        try:
+            return super(ImportOrigDebianGitRepository, self).merge(*args, **kwargs)
+        except GitRepositoryError:
+            # Only cleanup in the error case to undo working copy
+            # changes. Resetting the refs handles the other cases.
+            self.rrr_merge(commit)
+            raise
 
 
 def prepare_pristine_tar(archive, pkg, version):
@@ -58,7 +189,7 @@ def prepare_pristine_tar(archive, pkg, version):
         return None
 
     ext = os.path.splitext(archive)[1]
-    if ext in ['.tgz', '.tbz2', '.tlz', '.txz' ]:
+    if ext in ['.tgz', '.tbz2', '.tlz', '.txz']:
         ext = ".%s" % ext[2:]
 
     link = "../%s_%s.orig.tar%s" % (pkg, version, ext)
@@ -124,12 +255,12 @@ def detect_name_and_version(repo, source, options):
 
 
 def find_source(use_uscan, args):
-    """Find the tarball to import - either via uscan or via command line argument
+    """Find the main tarball to import - either via uscan or via command line argument
     @return: upstream source filename or None if nothing to import
     @rtype: string
     @raise GbpError: raised on all detected errors
 
-    >>> find_source(False, ['too', 'much'])
+    >>> find_source(False, ['too', 'many'])
     Traceback (most recent call last):
     ...
     GbpError: More than one archive specified. Try --help.
@@ -164,13 +295,12 @@ def find_source(use_uscan, args):
         else:
             gbp.log.info("package is up to date, nothing to do.")
             return None
-    if len(args) > 1: # source specified
+    if len(args) > 1:  # source specified
         raise GbpError("More than one archive specified. Try --help.")
     elif len(args) == 0:
         raise GbpError("No archive to import specified. Try --help.")
     else:
-        archive = DebianUpstreamSource(args[0])
-        return archive
+        return DebianUpstreamSource(args[0])
 
 
 def debian_branch_merge(repo, tag, version, options):
@@ -188,9 +318,16 @@ def debian_branch_merge(repo, tag, version, options):
             cp = ChangeLog(filename='debian/changelog')
             if cp.has_epoch():
                 epoch = '%s:' % cp.epoch
-        info = {'version': "%s%s-1" % (epoch, version)}
-        env = {'GBP_BRANCH': options.debian_branch}
-        gbpc.Command(format_str(options.postimport, info), extra_env=env, shell=True)()
+        debian_version = "%s%s-1" % (epoch, version)
+        info = {'version': debian_version}
+        env = {'GBP_BRANCH': options.debian_branch,
+               'GBP_TAG': tag,
+               'GBP_UPSTREAM_VERSION': version,
+               'GBP_DEBIAN_VERSION': debian_version,
+               }
+        Hook('Postimport',
+             format_str(options.postimport, info),
+             extra_env=env)()
 
 
 def debian_branch_merge_by_replace(repo, tag, version, options):
@@ -218,21 +355,42 @@ def debian_branch_merge_by_replace(repo, tag, version, options):
     repo.force_head(commit, hard=True)
 
 
+def get_component_tarballs(name, version, tarball, components):
+    """
+    Figure out the paths to the component tarballs based on the main
+    tarball.
+    """
+    tarballs = []
+    for component in components:
+        (_, _, comp_type) = parse_archive_filename(tarball)
+        cname = DebianPkgPolicy.build_tarball_name(name,
+                                                   version,
+                                                   comp_type,
+                                                   os.path.dirname(tarball),
+                                                   component)
+        tarballs.append((component, cname))
+        if not os.path.exists(cname):
+            raise GbpError("Can not find component tarball %s" % cname)
+    return tarballs
+
+
 def debian_branch_merge_by_merge(repo, tag, version, options):
     gbp.log.info("Merging to '%s'" % options.debian_branch)
+    branch = repo.get_branch()
+    repo.set_branch(options.debian_branch)
     try:
         repo.merge(tag)
     except GitRepositoryError:
-        raise GbpError("Merge failed, please resolve.")
-    repo.set_branch(options.debian_branch)
+        raise GbpError("Automatic merge failed.")
+    repo.set_branch(branch)
 
 
 def set_bare_repo_options(options):
     """Modify options for import into a bare repository"""
     if options.pristine_tar or options.merge:
         gbp.log.info("Bare repository: setting %s%s options"
-                      % (["", " '--no-pristine-tar'"][options.pristine_tar],
-                         ["", " '--no-merge'"][options.merge]))
+                     % (["", " '--no-pristine-tar'"][options.pristine_tar],
+                        ["", " '--no-merge'"][options.merge]))
         options.pristine_tar = False
         options.merge = False
 
@@ -241,52 +399,56 @@ def build_parser(name):
     try:
         parser = GbpOptionParserDebian(command=os.path.basename(name), prefix='',
                                        usage='%prog [options] /path/to/upstream-version.tar.gz | --uscan')
-    except configparser.ParsingError as err:
+    except GbpError as err:
         gbp.log.err(err)
         return None
 
     import_group = GbpOptionGroup(parser, "import options",
-                      "pristine-tar and filtering")
+                                  "pristine-tar and filtering")
     tag_group = GbpOptionGroup(parser, "tag options",
-                      "options related to git tag creation")
+                               "options related to git tag creation")
     branch_group = GbpOptionGroup(parser, "version and branch naming options",
-                      "version number and branch layout options")
-    cmd_group = GbpOptionGroup(parser, "external command options", "how and when to invoke external commands and hooks")
-
-    for group in [import_group, branch_group, tag_group, cmd_group ]:
+                                  "version number and branch layout options")
+    cmd_group = GbpOptionGroup(parser, "external command options",
+                               "how and when to invoke external commands and hooks")
+    for group in [import_group, branch_group, tag_group, cmd_group]:
         parser.add_option_group(group)
 
     branch_group.add_option("-u", "--upstream-version", dest="version",
-                      help="Upstream Version")
+                            help="Upstream Version")
     branch_group.add_config_file_option(option_name="debian-branch",
-                      dest="debian_branch")
+                                        dest="debian_branch")
     branch_group.add_config_file_option(option_name="upstream-branch",
-                      dest="upstream_branch")
+                                        dest="upstream_branch")
     branch_group.add_config_file_option(option_name="upstream-vcs-tag", dest="vcs_tag",
-                            help="Upstream VCS tag add to the merge commit")
+                                        help="Upstream VCS tag add to the merge commit")
     branch_group.add_boolean_config_file_option(option_name="merge", dest="merge")
     branch_group.add_config_file_option(option_name="merge-mode", dest="merge_mode")
 
     tag_group.add_boolean_config_file_option(option_name="sign-tags",
-                      dest="sign_tags")
+                                             dest="sign_tags")
     tag_group.add_config_file_option(option_name="keyid",
-                      dest="keyid")
+                                     dest="keyid")
     tag_group.add_config_file_option(option_name="upstream-tag",
-                      dest="upstream_tag")
+                                     dest="upstream_tag")
     import_group.add_config_file_option(option_name="filter",
-                      dest="filters", action="append")
+                                        dest="filters", action="append")
     import_group.add_boolean_config_file_option(option_name="pristine-tar",
-                      dest="pristine_tar")
+                                                dest="pristine_tar")
     import_group.add_boolean_config_file_option(option_name="filter-pristine-tar",
-                      dest="filter_pristine_tar")
+                                                dest="filter_pristine_tar")
     import_group.add_config_file_option(option_name="import-msg",
-                      dest="import_msg")
+                                        dest="import_msg")
     import_group.add_boolean_config_file_option(option_name="symlink-orig",
                                                 dest="symlink_orig")
+    import_group.add_config_file_option("component", action="append", metavar='COMPONENT',
+                                        dest="components")
     cmd_group.add_config_file_option(option_name="postimport", dest="postimport")
 
     parser.add_boolean_config_file_option(option_name="interactive",
                                           dest='interactive')
+    parser.add_boolean_config_file_option(option_name="rollback",
+                                          dest="rollback")
     parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False,
                       help="verbose command execution")
     parser.add_config_file_option(option_name="color", dest="color", type='tristate')
@@ -343,20 +505,19 @@ def main(argv):
     tmpdir = ''
     pristine_orig = None
     linked = False
+    repo = None
 
     (options, args) = parse_args(argv)
     if not options:
-        return 1
+        return ExitCodes.parse_error
 
     try:
         try:
-            repo = DebianGitRepository('.')
+            repo = ImportOrigDebianGitRepository('.')
         except GitRepositoryError:
             raise GbpError("%s is not a git repository" % (os.path.abspath('.')))
 
-        # an empty repo has now branches:
-        initial_branch = repo.get_branch()
-        is_empty = False if initial_branch else True
+        is_empty = repo.is_empty()
 
         if not repo.has_branch(options.upstream_branch) and not is_empty:
             raise GbpError(no_upstream_branch_msg % options.upstream_branch)
@@ -366,7 +527,7 @@ def main(argv):
             gbp.log.err("Repository has uncommitted changes, commit these first: ")
             raise GbpError(out)
 
-        # Download the source
+        # Download the main tarball
         if options.download:
             source = download_orig(args[0])
         else:
@@ -374,7 +535,13 @@ def main(argv):
         if not source:
             return ret
 
+        # The main tarball
         (sourcepackage, version) = detect_name_and_version(repo, source, options)
+        # Additionl tarballs we expect to exist
+        component_tarballs = get_component_tarballs(sourcepackage,
+                                                    version,
+                                                    source.path,
+                                                    options.components)
 
         tag = repo.version_to_tag(options.upstream_tag, version)
         if repo.has_tag(tag):
@@ -387,10 +554,12 @@ def main(argv):
             tmpdir = tempfile.mkdtemp(dir='../')
             source.unpack(tmpdir, options.filters)
             gbp.log.debug("Unpacked '%s' to '%s'" % (source.path, source.unpacked))
+            for (component, tarball) in component_tarballs:
+                unpack_component_tarball(source.unpacked, component, tarball, options.filters)
 
         if orig_needs_repack(source, options):
             gbp.log.debug("Filter pristine-tar: repacking '%s' from '%s'" % (source.path, source.unpacked))
-            (source, tmpdir)  = repack_source(source, sourcepackage, version, tmpdir, options.filters)
+            (source, tmpdir) = repack_source(source, sourcepackage, version, tmpdir, options.filters)
 
         (pristine_orig, linked) = prepare_pristine_tar(source.path,
                                                        sourcepackage,
@@ -404,32 +573,30 @@ def main(argv):
             pass
 
         try:
-            upstream_branch = [ options.upstream_branch, 'master' ][is_empty]
+            import_branch = options.upstream_branch
             filter_msg = ["", " (filtering out %s)"
                               % options.filters][len(options.filters) > 0]
             gbp.log.info("Importing '%s' to branch '%s'%s..." % (source.path,
-                                                                 upstream_branch,
+                                                                 import_branch,
                                                                  filter_msg))
             gbp.log.info("Source package is %s" % sourcepackage)
             gbp.log.info("Upstream version is %s" % version)
 
-            import_branch = [ options.upstream_branch, None ][is_empty]
             msg = upstream_import_commit_msg(options, version)
 
-            if options.vcs_tag:
-                parents = [repo.rev_parse("%s^{}" % repo.version_to_tag(options.vcs_tag, version))]
-            else:
-                parents = None
-
             commit = repo.commit_dir(source.unpacked,
                                      msg=msg,
                                      branch=import_branch,
-                                     other_parents=parents,
+                                     other_parents=repo.vcs_tag_parent(options.vcs_tag, version),
+                                     create_missing_branch=is_empty,
                                      )
 
             if options.pristine_tar:
                 if pristine_orig:
-                    repo.pristine_tar.commit(pristine_orig, upstream_branch)
+                    repo.rrr_branch('pristine-tar')
+                    repo.create_pristinetar_commits(import_branch,
+                                                    pristine_orig,
+                                                    component_tarballs)
                 else:
                     gbp.log.warn("'%s' not an archive, skipping pristine-tar" % source.path)
 
@@ -438,19 +605,19 @@ def main(argv):
                             commit=commit,
                             sign=options.sign_tags,
                             keyid=options.keyid)
+
             if is_empty:
-                repo.create_branch(options.upstream_branch, rev=commit)
-                repo.force_head(options.upstream_branch, hard=True)
-                if options.debian_branch != 'master':
-                    repo.rename_branch('master', options.debian_branch)
+                repo.create_branch(branch=options.debian_branch, rev=commit)
+                repo.force_head(options.debian_branch, hard=True)
             elif options.merge:
+                repo.rrr_branch(options.debian_branch)
                 debian_branch_merge(repo, tag, version, options)
 
             # Update working copy and index if we've possibly updated the
             # checked out branch
             current_branch = repo.get_branch()
-            if current_branch in [ options.upstream_branch,
-                                   repo.pristine_tar_branch]:
+            if current_branch in [options.upstream_branch,
+                                  repo.pristine_tar_branch]:
                 repo.force_head(current_branch, hard=True)
         except (gbpc.CommandExecFailed, GitRepositoryError) as err:
             msg = str(err) or 'Unknown error, please report a bug'
@@ -459,6 +626,16 @@ def main(argv):
         if str(err):
             gbp.log.err(err)
         ret = 1
+        if repo and repo.has_rollbacks() and options.rollback:
+            gbp.log.err("Error detected, Will roll back changes.")
+            try:
+                repo.rollback()
+                # Make sure the very last line as an error message
+                gbp.log.err("Rolled back changes after import error.")
+            except Exception as e:
+                gbp.log.error("%s" % e)
+                gbp.log.error("Clean up manually and please report a bug: %s" %
+                              repo.rollback_errors)
 
     if pristine_orig and linked and not options.symlink_orig:
         os.unlink(pristine_orig)
