@@ -17,7 +17,6 @@
 #
 """Manage Debian patches on a patch queue branch"""
 
-from six.moves import configparser
 import errno
 import os
 import shutil
@@ -25,20 +24,23 @@ import sys
 import tempfile
 import re
 from gbp.config import GbpOptionParserDebian
-from gbp.git import (GitRepositoryError, GitRepository)
+from gbp.deb.source import DebianSource
+from gbp.deb.git import DebianGitRepository
+from gbp.git import GitRepositoryError
 from gbp.command_wrappers import (GitCommand, CommandExecFailed)
 from gbp.errors import GbpError
 import gbp.log
 from gbp.patch_series import (PatchSeries, Patch)
 from gbp.scripts.common.pq import (is_pq_branch, pq_branch_name, pq_branch_base,
-                                 parse_gbp_commands, format_patch,
-                                 switch_to_pq_branch, apply_single_patch,
-                                 apply_and_commit_patch, switch_pq,
-                                 drop_pq, get_maintainer_from_control)
+                                   parse_gbp_commands, format_patch,
+                                   switch_to_pq_branch, apply_single_patch,
+                                   apply_and_commit_patch, switch_pq,
+                                   drop_pq, get_maintainer_from_control)
+from gbp.scripts.common import ExitCodes
 from gbp.dch import extract_bts_cmds
 
 PATCH_DIR = "debian/patches/"
-SERIES_FILE = os.path.join(PATCH_DIR,"series")
+SERIES_FILE = os.path.join(PATCH_DIR, "series")
 
 
 def parse_old_style_topic(commit_info):
@@ -145,7 +147,7 @@ def format_series_diff(added, removed, options):
     return msg
 
 
-def commit_patches(repo, branch, patches, options):
+def commit_patches(repo, branch, patches, options, patch_dir):
     """
     Commit chanages exported from patch queue
     """
@@ -156,12 +158,12 @@ def commit_patches(repo, branch, patches, options):
     vfs = gbp.git.vfs.GitVfs(repo, branch)
     try:
         oldseries = vfs.open('debian/patches/series')
-        oldpatches = [ p.strip() for p in oldseries.readlines() ]
+        oldpatches = [p.strip() for p in oldseries.readlines()]
         oldseries.close()
     except IOError:
         # No series file yet
         oldpatches = []
-    newpatches = [ p[len(PATCH_DIR):] for p in patches ]
+    newpatches = [p[len(patch_dir):] for p in patches]
 
     # FIXME: handle case were only the contents of the patches changed
     added, removed = compare_series(oldpatches, newpatches)
@@ -171,8 +173,29 @@ def commit_patches(repo, branch, patches, options):
     return added, removed
 
 
+def find_upstream_commit(repo, branch, upstream_tag):
+    """
+    Find commit corresponding upstream version based on changelog
+    """
+    vfs = gbp.git.vfs.GitVfs(repo, branch)
+    cl = DebianSource(vfs).changelog
+    upstream_commit = repo.find_version(upstream_tag, cl.upstream_version)
+    if not upstream_commit:
+        raise GbpError("Couldn't find upstream version %s" %
+                       cl.upstream_version)
+    return upstream_commit
+
+
+def pq_on_upstream_tag(pq_from):
+    """Return True if the patch queue is based on the uptream tag,
+    False if its based on the debian packaging branch"""
+    return True if pq_from.upper() == 'TAG' else False
+
+
 def export_patches(repo, branch, options):
     """Export patches from the pq branch into a patch series"""
+    patch_dir = os.path.join(repo.path, PATCH_DIR)
+    series_file = os.path.join(repo.path, SERIES_FILE)
     if is_pq_branch(branch):
         base = pq_branch_base(branch)
         gbp.log.info("On '%s', switching to '%s'" % (branch, base))
@@ -181,21 +204,26 @@ def export_patches(repo, branch, options):
 
     pq_branch = pq_branch_name(branch)
     try:
-        shutil.rmtree(PATCH_DIR)
+        shutil.rmtree(patch_dir)
     except OSError as e:
         if e.errno != errno.ENOENT:
             raise GbpError("Failed to remove patch dir: %s" % e.strerror)
         else:
-            gbp.log.debug("%s does not exist." % PATCH_DIR)
+            gbp.log.debug("%s does not exist." % patch_dir)
 
-    patches = generate_patches(repo, branch, pq_branch, PATCH_DIR, options)
+    if pq_on_upstream_tag(options.pq_from):
+        base = find_upstream_commit(repo, branch, options.upstream_tag)
+    else:
+        base = branch
+
+    patches = generate_patches(repo, base, pq_branch, patch_dir, options)
 
     if patches:
-        with open(SERIES_FILE, 'w') as seriesfd:
+        with open(series_file, 'w') as seriesfd:
             for patch in patches:
-                seriesfd.write(os.path.relpath(patch, PATCH_DIR) + '\n')
+                seriesfd.write(os.path.relpath(patch, patch_dir) + '\n')
         if options.commit:
-            added, removed = commit_patches(repo, branch, patches, options)
+            added, removed = commit_patches(repo, branch, patches, options, patch_dir)
             if added:
                 what = 'patches' if len(added) > 1 else 'patch'
                 gbp.log.info("Added %s %s to patch series" % (what, ', '.join(added)))
@@ -203,7 +231,7 @@ def export_patches(repo, branch, options):
                 what = 'patches' if len(removed) > 1 else 'patch'
                 gbp.log.info("Removed %s %s from patch series" % (what, ', '.join(removed)))
         else:
-            GitCommand('status')(['--', PATCH_DIR])
+            GitCommand('status', cwd=repo.path)(['--', PATCH_DIR])
     else:
         gbp.log.info("No patches on '%s' - nothing to do." % pq_branch)
 
@@ -211,7 +239,7 @@ def export_patches(repo, branch, options):
         drop_pq(repo, branch)
 
 
-def safe_patches(series):
+def safe_patches(series, repo):
     """
     Safe the current patches in a temporary directory
     below .git/
@@ -224,7 +252,7 @@ def safe_patches(series):
     src = os.path.dirname(series)
     name = os.path.basename(series)
 
-    tmpdir = tempfile.mkdtemp(dir='.git/', prefix='gbp-pq')
+    tmpdir = tempfile.mkdtemp(dir=repo.git_dir, prefix='gbp-pq')
     patches = os.path.join(tmpdir, 'patches')
     series = os.path.join(patches, name)
 
@@ -234,19 +262,25 @@ def safe_patches(series):
     return (tmpdir, series)
 
 
-def import_quilt_patches(repo, branch, series, tries, force):
+def import_quilt_patches(repo, branch, series, tries, force, pq_from,
+                         upstream_tag):
     """
     apply a series of quilt patches in the series file 'series' to branch
     the patch-queue branch for 'branch'
 
     @param repo: git repository to work on
-    @param branch: branch to base pqtch queue on
-    @param series; series file to read patches from
+    @param branch: branch to base patch queue on
+    @param series: series file to read patches from
     @param tries: try that many times to apply the patches going back one
                   commit in the branches history after each failure.
     @param force: import the patch series even if the branch already exists
+    @param pq_from: what to use as the starting point for the pq branch.
+                    DEBIAN indicates the current branch, TAG indicates that
+                    the corresponding upstream tag should be used.
+    @param upstream_tag: upstream tag template to use
     """
     tmpdir = None
+    series = os.path.join(repo.path, series)
 
     if is_pq_branch(branch):
         if force:
@@ -267,11 +301,16 @@ def import_quilt_patches(repo, branch, series, tries, force):
                            % pq_branch)
 
     maintainer = get_maintainer_from_control(repo)
-    commits = repo.get_commits(num=tries, first_parent=True)
+    if pq_on_upstream_tag(pq_from):
+        commits = [find_upstream_commit(repo, branch, upstream_tag)]
+    else:  # pq_from == 'DEBIAN'
+        commits = repo.get_commits(num=tries, first_parent=True)
     # If we go back in history we have to safe our pq so we always try to apply
     # the latest one
-    if len(commits) > 1:
-        tmpdir, series = safe_patches(series)
+    # If we are using the upstream_tag, we always need a copy of the patches
+    if len(commits) > 1 or pq_on_upstream_tag(pq_from):
+        if os.path.exists(series):
+            tmpdir, series = safe_patches(series, repo)
 
     queue = PatchSeries.read_series_file(series)
 
@@ -300,7 +339,7 @@ def import_quilt_patches(repo, branch, series, tries, force):
         else:
             # All patches applied successfully
             break
-        i-=1
+        i -= 1
     else:
         raise GbpError("Couldn't apply patches")
 
@@ -308,31 +347,44 @@ def import_quilt_patches(repo, branch, series, tries, force):
         gbp.log.debug("Remove temporary patch safe '%s'" % tmpdir)
         shutil.rmtree(tmpdir)
 
+    return len(queue)
+
+
+def rebase_pq(repo, branch, pq_from, upstream_tag):
 
-def rebase_pq(repo, branch):
     if is_pq_branch(branch):
         base = pq_branch_base(branch)
     else:
         switch_to_pq_branch(repo, branch)
         base = branch
-    GitCommand("rebase")([base])
+
+    if pq_on_upstream_tag(pq_from):
+        _from = find_upstream_commit(repo, base, upstream_tag)
+    else:
+        _from = base
+
+    GitCommand("rebase", cwd=repo.path)([_from])
+
+
+def usage_msg():
+    return """%prog [options] action - maintain patches on a patch queue branch
+Actions:
+  export         export the patch queue associated to the current branch
+                 into a quilt patch series in debian/patches/ and update the
+                 series file.
+  import         create a patch queue branch from quilt patches in debian/patches.
+  rebase         switch to patch queue branch associated to the current
+                 branch and rebase against current branch.
+  drop           drop (delete) the patch queue associated to the current branch.
+  apply          apply a patch
+  switch         switch to patch-queue branch and vice versa"""
 
 
 def build_parser(name):
     try:
         parser = GbpOptionParserDebian(command=os.path.basename(name),
-                                   usage="%prog [options] action - maintain patches on a patch queue branch\n"
-        "Actions:\n"
-        "  export         export the patch queue associated to the current branch\n"
-        "                 into a quilt patch series in debian/patches/ and update the\n"
-        "                 series file.\n"
-        "  import         create a patch queue branch from quilt patches in debian/patches.\n"
-        "  rebase         switch to patch queue branch associated to the current\n"
-        "                 branch and rebase against current branch.\n"
-        "  drop           drop (delete) the patch queue associated to the current branch.\n"
-        "  apply          apply a patch\n"
-        "  switch         switch to patch-queue branch and vice versa")
-    except configparser.ParsingError as err:
+                                       usage=usage_msg())
+    except GbpError as err:
         gbp.log.err(err)
         return None
 
@@ -352,6 +404,8 @@ def build_parser(name):
                                   dest="color_scheme")
     parser.add_config_file_option(option_name="meta-closes", dest="meta_closes")
     parser.add_config_file_option(option_name="meta-closes-bugnum", dest="meta_closes_bugnum")
+    parser.add_config_file_option(option_name="pq-from", dest="pq_from", choices=['DEBIAN', 'TAG'])
+    parser.add_config_file_option(option_name="upstream-tag", dest="upstream_tag")
     return parser
 
 
@@ -367,7 +421,7 @@ def main(argv):
 
     (options, args) = parse_args(argv)
     if not options:
-        return 1
+        return ExitCodes.parse_error
 
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
 
@@ -390,7 +444,7 @@ def main(argv):
         return 1
 
     try:
-        repo = GitRepository(os.path.curdir)
+        repo = DebianGitRepository(os.path.curdir)
     except GitRepositoryError:
         gbp.log.err("%s is not a git repository" % (os.path.abspath('.')))
         return 1
@@ -402,20 +456,25 @@ def main(argv):
         elif action == "import":
             series = SERIES_FILE
             tries = options.time_machine if (options.time_machine > 0) else 1
-            import_quilt_patches(repo, current, series, tries, options.force)
+            num = import_quilt_patches(repo, current, series, tries,
+                                       options.force, options.pq_from,
+                                       options.upstream_tag)
             current = repo.get_branch()
-            gbp.log.info("Patches listed in '%s' imported on '%s'" %
-                          (series, current))
+            gbp.log.info("%d patches listed in '%s' imported on '%s'" %
+                         (num, series, current))
         elif action == "drop":
             drop_pq(repo, current)
         elif action == "rebase":
-            rebase_pq(repo, current)
+            rebase_pq(repo, current, options.pq_from, options.upstream_tag)
         elif action == "apply":
             patch = Patch(patchfile)
             maintainer = get_maintainer_from_control(repo)
             apply_single_patch(repo, current, patch, maintainer, options.topic)
         elif action == "switch":
             switch_pq(repo, current)
+    except KeyboardInterrupt:
+        retval = 1
+        gbp.log.err("Interrupted. Aborting.")
     except CommandExecFailed:
         retval = 1
     except (GbpError, GitRepositoryError) as err:
