@@ -5,7 +5,8 @@ Test L{gbp.config.GbpOptionParser}
 Test L{gbp.config.GbpOptionParserDebian}
 """
 
-from . import context
+from . import context  # noqa: F401
+
 
 def test_option_parser():
     """
@@ -20,6 +21,7 @@ def test_option_parser():
     >>> c.add_boolean_config_file_option(option_name='track', dest='track')
     """
 
+
 def test_option_parser_debian():
     """
     Methods tested:
@@ -34,6 +36,7 @@ def test_option_parser_debian():
     >>> c.add_config_file_option(option_name='builder', dest='builder', help='foo')
     """
 
+
 def test_option_group():
     """
     Methods tested:
@@ -47,11 +50,12 @@ def test_option_group():
     >>> g.add_boolean_config_file_option(option_name='track', dest='track')
     """
 
+
 def test_tristate():
     """
     Methods tested:
          - L{gbp.config.GbpOptionParser.add_config_file_option}
-    
+
     >>> import gbp.config
     >>> c = gbp.config.GbpOptionParser('tristate')
     >>> c.add_config_file_option(option_name="color", dest="color", type='tristate')
@@ -60,6 +64,7 @@ def test_tristate():
     auto
     """
 
+
 def test_filter():
     """
     The filter option should always parse as a list
@@ -80,3 +85,19 @@ def test_filter():
     ['this', 'is', 'a', 'list']
     >>> del os.environ['GBP_CONF_FILES']
     """
+
+
+def test_filters():
+    """
+    The filter can be given in plural form
+    >>> import os
+    >>> from gbp.config import GbpOptionParser
+    >>> tmpdir = str(context.new_tmpdir('bar'))
+    >>> confname = os.path.join(tmpdir, 'gbp.conf')
+    >>> GbpOptionParser._set_config_file_value('bar', 'filters', '["abc", "def"]\\n', filename=confname)
+    >>> os.environ['GBP_CONF_FILES'] = confname
+    >>> parser = GbpOptionParser('bar')
+    >>> parser.config['filter']
+    ['abc', 'def']
+    >>> del os.environ['GBP_CONF_FILES']
+    """
