@@ -5,10 +5,10 @@ Test L{gbp.git.GitRepository}
 
 This testcase creates several repositores:
 
-    - A repository at L{repo_dir} called I{repo}
-    - A bare repository at L{bare_dir} called I{bare}
-    - A clone of I{repo} below L{clone_dir} called I{clone}
-    - A mirror of I{repo} below L{mirror_clone_dir} called I{mirror}
+    - A repository at I{dirs['repo']} called I{repo}
+    - A bare repository at I{dirs['bare']} called I{bare}
+    - A clone of I{repo} below I{dirs['clone']} called I{clone}
+    - A mirror of I{repo} below I{mirror_dirs['clone']} called I{mirror}
 """
 
 from . import context
@@ -17,9 +17,21 @@ import gbp.log
 
 gbp.log.setup(color=False, verbose=True)
 
-repo_dir, bare_dir, clone_dir, mirror_clone_dir = list(map(
-    lambda x, tmpdir=context.new_tmpdir(__name__): tmpdir.join(x),
-    ['repo', 'bare', 'clone', 'mirror_clone']))
+dirs = {}
+subdirs = ['repo', 'bare', 'clone', 'mirror_clone']
+
+
+def setup_module():
+    tmpdir = context.new_tmpdir(__name__)
+    for s in subdirs:
+        dirs[s] = tmpdir.join(s)
+
+
+def teardown_module():
+    for s in subdirs:
+        del dirs[s]
+    context.teardown()
+
 
 def test_create():
     """
@@ -33,10 +45,10 @@ def test_create():
          - L{gbp.git.GitRepository.git_dir}
 
     >>> import os, gbp.git
-    >>> repo = gbp.git.GitRepository.create(repo_dir)
-    >>> repo.path == repo_dir
+    >>> repo = gbp.git.GitRepository.create(dirs['repo'])
+    >>> repo.path == dirs['repo']
     True
-    >>> repo.git_dir == os.path.join(repo_dir, '.git')
+    >>> repo.git_dir == os.path.join(dirs['repo'], '.git')
     True
     >>> type(repo) == gbp.git.GitRepository
     True
@@ -52,7 +64,7 @@ def test_empty():
          - L{gbp.git.GitRepository.is_empty}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.get_branch()
     >>> repo.branch
     >>> repo.is_empty()
@@ -73,7 +85,7 @@ def test_add_files():
          - L{gbp.git.GitRepository.head}
 
     >>> import gbp.git, shutil, os
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> ret = shutil.copy(os.path.join(repo.path, ".git/HEAD"),
     ...                                os.path.join(repo.path, "testfile"))
     >>> repo.is_clean()[0]
@@ -97,13 +109,14 @@ def test_branch_master():
     Methods tested:
          - L{gbp.git.GitRepository.get_branch}
     >>> import gbp.git, shutil
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.get_branch()
     'master'
     >>> repo.branch
     'master'
     """
 
+
 def test_clean():
     """
     Remove untracked files from the working tree
@@ -112,7 +125,7 @@ def test_clean():
          - L{gbp.git.GitRepository.clean}
 
     >>> import gbp.git, shutil, os
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> ret = shutil.copy(os.path.join(repo.path, ".git/HEAD"),
     ...                                os.path.join(repo.path, "testclean"))
     >>> repo.clean(dry_run=True)
@@ -123,6 +136,7 @@ def test_clean():
     True
     """
 
+
 def test_create_branch():
     """
     Create a branch name I{foo}
@@ -132,7 +146,7 @@ def test_create_branch():
          - L{gbp.git.GitRepository.branch_contains}
 
     >>> import gbp.git, shutil
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.create_branch("foo")
     >>> repo.branch_contains("foo", 'HEAD')
     True
@@ -140,6 +154,7 @@ def test_create_branch():
     False
     """
 
+
 def test_delete_branch():
     """
     Create a branch named I{foo2} and delete it
@@ -149,7 +164,7 @@ def test_delete_branch():
          - L{gbp.git.GitRepository.delete_branch}
 
     >>> import gbp.git, shutil
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.create_branch("bar")
     >>> repo.delete_branch("bar")
     >>> repo.delete_branch("master")
@@ -158,6 +173,7 @@ def test_delete_branch():
     GitRepositoryError: Can't delete the branch you're on
     """
 
+
 def test_set_branch():
     """
     Switch to branch named I{foo}
@@ -168,7 +184,7 @@ def test_set_branch():
          - L{gbp.git.GitRepository.branch}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.set_branch("foo")
     >>> repo.get_branch() == "foo"
     True
@@ -187,7 +203,7 @@ def test_rename_branch():
          - L{gbp.git.GitRepository.delete_branch}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.create_branch("baz")
     >>> repo.rename_branch("baz", "bax")
     >>> repo.delete_branch("bax")
@@ -200,7 +216,7 @@ def test_set_upstream_branch():
 
     >>> import os, shutil
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> os.makedirs(os.path.join(repo.git_dir, 'refs/remotes/origin'))
     >>> ret = shutil.copy(os.path.join(repo.git_dir, 'refs/heads/master'), \
                     os.path.join(repo.git_dir, 'refs/remotes/origin/'))
@@ -216,12 +232,13 @@ def test_set_upstream_branch():
     GitRepositoryError: Branch origin/bla doesn't exist!
     """
 
+
 def test_get_upstream_branch():
     """
     Get info about upstream branches set in test_set_upstream_branch
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.get_upstream_branch('master')
     'origin/master'
     >>> repo.get_upstream_branch('foo')
@@ -231,6 +248,7 @@ def test_get_upstream_branch():
     GitRepositoryError: Branch bla doesn't exist!
     """
 
+
 def test_tag():
     """
     Create a tag named I{tag} and check its existance
@@ -242,7 +260,7 @@ def test_tag():
          - L{gbp.git.GitRepository.get_tags}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.create_tag("tag")
     >>> repo.has_tag("tag")
     True
@@ -259,6 +277,7 @@ def test_tag():
     ['tag', 'tag2']
     """
 
+
 def test_describe():
     """
     Describe commit-ish
@@ -267,7 +286,7 @@ def test_describe():
          - L{gbp.git.GitRepository.describe}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> sha = repo.rev_parse('HEAD')
     >>> repo.describe('HEAD')
     'tag2'
@@ -292,6 +311,7 @@ def test_describe():
     >>> repo.create_tag('tag2', msg='foo')
     """
 
+
 def test_find_tag():
     """
     Find tags
@@ -300,7 +320,7 @@ def test_find_tag():
          - L{gbp.git.GitRepository.find_tag}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.find_tag('HEAD')
     'tag2'
     >>> repo.find_tag('HEAD', pattern='foo*')
@@ -309,6 +329,7 @@ def test_find_tag():
     GitRepositoryError: Can't describe HEAD. Git error: fatal: No names found, cannot describe anything.
     """
 
+
 def test_find_branch_tag():
     """
     Find the closest tags on a certain branch to a given commit
@@ -317,7 +338,7 @@ def test_find_branch_tag():
          - L{gbp.git.GitRepository.find_branch_tag}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.find_branch_tag('HEAD', 'master', 'tag*')
     'tag2'
     >>> repo.find_branch_tag('HEAD', 'master', 'v*')   # doctest:+ELLIPSIS
@@ -326,6 +347,7 @@ def test_find_branch_tag():
     GitRepositoryError: Can't describe .... Git error: fatal: No names found, cannot describe anything.
     """
 
+
 def test_move_tag():
     """
     Move a tag
@@ -335,7 +357,7 @@ def test_move_tag():
          - L{gbp.git.GitRepository.has_tag}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.move_tag("tag", "moved")
     >>> repo.has_tag("tag")
     False
@@ -343,6 +365,7 @@ def test_move_tag():
     True
     """
 
+
 def test_delete_tag():
     """
     Delete tags
@@ -352,7 +375,7 @@ def test_delete_tag():
          - L{gbp.git.GitRepository.has_tag}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.has_tag("moved")
     True
     >>> repo.delete_tag("moved")
@@ -360,6 +383,7 @@ def test_delete_tag():
     False
     """
 
+
 def test_get_obj_type():
     """
     Find commit SHA1 related to tags
@@ -370,7 +394,7 @@ def test_get_obj_type():
          - L{gbp.git.GitRepository.delete_tag}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.create_tag("tag3", "tag msg")
     >>> repo.get_obj_type("tag3")
     'tag'
@@ -381,6 +405,7 @@ def test_get_obj_type():
     >>> repo.delete_tag("tag3")
     """
 
+
 def test_list_files():
     """
     List files in the index
@@ -393,7 +418,7 @@ def test_list_files():
          - L{gbp.git.GitRepository.force_head}
 
     >>> import gbp.git, os, shutil
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> src = os.path.join(repo.path, ".git/HEAD")
     >>> dst = os.path.join(repo.path, "testfile")
     >>> repo.list_files()
@@ -426,6 +451,7 @@ def test_list_files():
     []
     """
 
+
 def test_get_commits():
     """
     Test listing commits
@@ -434,7 +460,7 @@ def test_get_commits():
          - L{gbp.git.GitRepository.get_commits}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> commits = repo.get_commits()
     >>> type(commits) == list and len(commits) == 2
     True
@@ -466,7 +492,7 @@ def test_get_commit_info():
 
     >>> import gbp.git
     >>> from datetime import datetime
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> info = repo.get_commit_info('HEAD')
     >>> info['id']
     'HEAD'
@@ -491,6 +517,7 @@ def test_get_commit_info():
     'foo'
     """
 
+
 def test_diff():
     """
     Test git-diff
@@ -499,7 +526,7 @@ def test_diff():
          - L{gbp.git.GitRepository.diff}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> len(repo.diff('HEAD~1', 'HEAD')) > 3
     True
     >>> len(repo.diff('HEAD~1', 'HEAD', 'testfile')) > 3
@@ -512,19 +539,21 @@ def test_diff():
     True
     """
 
+
 def test_diff_status():
     """
     Methods tested:
         - L{gbp.git.GitRepository.diff_status}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.diff_status("HEAD", "HEAD")            # doctest:+ELLIPSIS
     defaultdict(<... 'list'>, {})
     >>> repo.diff_status("HEAD~1", "HEAD")          # doctest:+ELLIPSIS
     defaultdict(<... 'list'>, {'M': ['testfile']})
     """
 
+
 def test_mirror_clone():
     """
     Mirror a repository
@@ -537,9 +566,9 @@ def test_mirror_clone():
          - L{gbp.git.GitRepository.branch}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.set_branch('master')
-    >>> mirror = gbp.git.GitRepository.clone(mirror_clone_dir, repo.path, mirror=True)
+    >>> mirror = gbp.git.GitRepository.clone(dirs['mirror_clone'], repo.path, mirror=True)
     >>> mirror.is_empty()
     False
     >>> mirror.branch
@@ -554,6 +583,7 @@ def test_mirror_clone():
     >>> mirror.force_head('foo^')
     """
 
+
 def test_clone():
     """
     Clone a repository
@@ -570,9 +600,9 @@ def test_clone():
          - L{gbp.git.GitRepository.has_remote_repo}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.set_branch('master')
-    >>> clone = gbp.git.GitRepository.clone(clone_dir, repo.path)
+    >>> clone = gbp.git.GitRepository.clone(dirs['clone'], repo.path)
     >>> clone.is_empty()
     False
     >>> clone.branch
@@ -600,6 +630,7 @@ def test_clone():
     False
     """
 
+
 def test_get_remotes():
     """
     Merge a branch
@@ -609,19 +640,20 @@ def test_get_remotes():
 
     >>> import os
     >>> import gbp.git.repository
-    >>> repo = gbp.git.repository.GitRepository(os.path.join(clone_dir, 'repo'))
+    >>> repo = gbp.git.repository.GitRepository(os.path.join(dirs['clone'], 'repo'))
     >>> remotes = repo.get_remotes()
     >>> len(remotes)
     1
     >>> origin = remotes['origin']
     >>> origin.name
     'origin'
-    >>> origin.fetch_url == repo_dir
+    >>> origin.fetch_url == dirs['repo']
     True
-    >>> origin.push_urls == [repo_dir]
+    >>> origin.push_urls == [dirs['repo']]
     True
     """
 
+
 def test_merge():
     """
     Merge a branch
@@ -631,11 +663,12 @@ def test_merge():
          - L{gbp.git.GitRepository.set_branch}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.set_branch('master')
     >>> repo.merge('foo')
     """
 
+
 def test_pull():
     """
     Pull from a remote repository
@@ -645,7 +678,7 @@ def test_pull():
          - L{gbp.git.GitRepository.pull}
 
     >>> import gbp.git, os
-    >>> d = os.path.join(clone_dir, 'repo')
+    >>> d = os.path.join(dirs['clone'], 'repo')
     >>> clone = gbp.git.GitRepository(d)
     >>> clone.set_branch('master')
     >>> clone.pull()
@@ -653,6 +686,7 @@ def test_pull():
     >>> clone.pull('origin', all_remotes=True)
     """
 
+
 def test_fetch():
     """
     Fetch from a remote repository
@@ -665,7 +699,7 @@ def test_fetch():
          - L{gbp.git.GitRepository.remove_remote_repo}
 
     >>> import gbp.git, os
-    >>> d = os.path.join(clone_dir, 'repo')
+    >>> d = os.path.join(dirs['clone'], 'repo')
     >>> clone = gbp.git.GitRepository(d)
     >>> clone.fetch()
     >>> clone.push()
@@ -676,7 +710,7 @@ def test_fetch():
     >>> clone.push_tag('origin', 'tag3')
     >>> clone.create_tag('tag4')
     >>> clone.push('origin', 'master', tags=True)
-    >>> clone.add_remote_repo('foo', repo_dir)
+    >>> clone.add_remote_repo('foo', dirs['repo'])
     >>> clone.fetch('foo')
     >>> clone.fetch('foo', tags=True)
     >>> clone.fetch('foo', refspec='refs/heads/master')
@@ -684,6 +718,7 @@ def test_fetch():
     >>> clone.remove_remote_repo('foo')
     """
 
+
 def test_create_bare():
     """
     Create a bare repository
@@ -693,10 +728,10 @@ def test_create_bare():
          - L{gbp.git.GitRepository.is_empty}
 
     >>> import gbp.git
-    >>> bare = gbp.git.GitRepository.create(bare_dir, bare=True, description="msg")
-    >>> bare.path == bare_dir
+    >>> bare = gbp.git.GitRepository.create(dirs['bare'], bare=True, description="msg")
+    >>> bare.path == dirs['bare']
     True
-    >>> bare.git_dir[:-1] == bare_dir
+    >>> bare.git_dir[:-1] == dirs['bare']
     True
     >>> type(bare) == gbp.git.GitRepository
     True
@@ -706,9 +741,10 @@ def test_create_bare():
     (True, '')
     """
 
-def test_nonexistant():
+
+def test_nonexistent():
     """
-    Check that accessing a non existant repository fails.
+    Check that accessing a non-existent repository fails.
 
     Methods tested:
          - L{gbp.git.GitRepository.__init__}
@@ -720,6 +756,7 @@ def test_nonexistant():
     GitRepositoryError: No Git repository at '/does/not/exist'
     """
 
+
 def test_create_noperm():
     """
     Check that creating a repository at a path that isn't writeable fails
@@ -734,6 +771,7 @@ def test_create_noperm():
     GitRepositoryError: Cannot create Git repository at '/does/not/exist': Permission denied
     """
 
+
 def test_checkout():
     """
     Checkout treeishs
@@ -749,7 +787,7 @@ def test_checkout():
          - L{gbp.git.GitRepository.tags}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.checkout('master')
     >>> repo.branch
     'master'
@@ -774,6 +812,7 @@ def test_checkout():
     >>> repo.branch
     """
 
+
 def test_gc():
     """
     Test garbace collection
@@ -782,10 +821,11 @@ def test_gc():
          - L{gbp.git.GitRepository.collect_garbage}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.collect_garbage()
     """
 
+
 def test_grep_log():
     """
     Test grepping through commit messages
@@ -794,7 +834,7 @@ def test_grep_log():
         - L{gbp.git.GitRepository.grep_log}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.set_branch('master')
     >>> len(repo.grep_log('foo')) == 2
     True
@@ -808,6 +848,7 @@ def test_grep_log():
     GitRepositoryError: Error grepping log for foo: fatal: bad revision 'doesnotexist'
     """
 
+
 def test_is_ff():
     """
     Test if branch is fast forwardable
@@ -816,7 +857,7 @@ def test_is_ff():
         - L{gbp.git.GitRepository.is_fast_forward}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.is_fast_forward('master', 'foo')
     (True, True)
     >>> repo.create_branch('ff', 'HEAD^')
@@ -826,6 +867,7 @@ def test_is_ff():
     (False, True)
     """
 
+
 def test_update_ref():
     """
     Test updating a reference
@@ -834,7 +876,7 @@ def test_update_ref():
         - L{gbp.git.GitRepository.update_ref}
 
     >>> import gbp.git, os
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.update_ref('new_ref', 'master', msg='update')
     >>> os.path.exists(os.path.join(repo.git_dir, 'new_ref'))
     True
@@ -851,7 +893,7 @@ def test_make_tree():
         - L{gbp.git.GitRepository.make_tree}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> sha1 = repo.write_file('testfile')
     >>> sha1
     '19af7398c894bc5e86e17259317e4db519e9241f'
@@ -876,12 +918,13 @@ def test_update_submodules():
         - L{gbp.git.GitRepository.update_submodules}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo.has_submodules()
     False
     >>> repo.update_submodules()
     """
 
+
 def test_get_merge_base():
     """
     Find the common ancestor of two objects
@@ -890,7 +933,7 @@ def test_get_merge_base():
         - L{gbp.git.GitRepository.get_merge_base}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> sha1 = repo.get_merge_base('master', 'foo')
     >>> len(sha1)
     40
@@ -900,13 +943,14 @@ def test_get_merge_base():
     GitRepositoryError: Failed to get common ancestor: fatal: Not a valid object name doesnotexist
     """
 
+
 def test_status():
     r"""
     Methods tested:
         - L{gbp.git.GitRepository.status}
 
     >>> import gbp.git, os, shutil
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> fname = os.path.join(repo.path, "test_status")
     >>> ret = shutil.copy(os.path.join(repo.path, ".git/HEAD"), fname)
     >>> list(repo.status().items())
@@ -922,13 +966,14 @@ def test_status():
     [('R ', ['test_status\x00test_statusnew'])]
     """
 
+
 def test_cmd_has_feature():
     r"""
     Methods tested:
         - L{gbp.git.GitRepository._cmd_has_feature}
 
     >>> import gbp.git
-    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo = gbp.git.GitRepository(dirs['repo'])
     >>> repo._cmd_has_feature("commit", "a")
     True
     >>> repo._cmd_has_feature("commit", "reuse-message")
@@ -951,11 +996,4 @@ def test_cmd_has_feature():
     True
     """
 
-def test_teardown():
-    """
-    Perform the teardown
-
-    >>> context.teardown()
-    """
-
 # vim:et:ts=4:sw=4:et:sts=4:ai:set list listchars=tab\:»·,trail\:·:
