@@ -1,5 +1,4 @@
 # vim: set fileencoding=utf-8 :
-
 """
 Test pristine-tar related methods in
 
@@ -11,16 +10,26 @@ and
 
 This testcase creates this reposity:
 
-    - A repository at L{repo_dir} called I{repo}
+    - A repository at I{dirs['repo']} called I{repo}
 
 """
 
+import os
 from . import context
 
-import os
 
-repo_dir = context.new_tmpdir(__name__).join('repo')
 test_data = os.path.join(context.projectdir, "tests/test_PristineTar_data")
+dirs = {}
+
+
+def setup_module():
+    dirs['repo'] = context.new_tmpdir(__name__).join('repo')
+
+
+def teardown_module():
+    del dirs['repo']
+    context.teardown()
+
 
 def test_create():
     """
@@ -30,9 +39,10 @@ def test_create():
          - L{gbp.deb.git.DebianGitRepository.create}
 
     >>> import os, gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository.create(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository.create(dirs['repo'])
     """
 
+
 def test_empty_repo():
     """
     Empty repos have no branch pristine-tar branch
@@ -42,13 +52,14 @@ def test_empty_repo():
          - L{gbp.deb.pristinetar.DebianPristineTar.has_commit}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.has_pristine_tar_branch()
     False
     >>> repo.pristine_tar.has_commit('upstream', '1.0', 'gzip')
     False
     """
 
+
 def test_commit_dir():
     """
     Empty repos have no branch pristine-tar branch
@@ -58,11 +69,12 @@ def test_commit_dir():
          - L{gbp.git.repository.GitRepository.create_branch}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> commit = repo.commit_dir(test_data, msg="initial commit", branch=None)
     >>> repo.create_branch('upstream')
     """
 
+
 def test_create_tarball():
     """
     Create a tarball from a git tree
@@ -71,11 +83,12 @@ def test_create_tarball():
          - L{gbp.deb.git.DebianGitRepository.archive}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.archive('tar', 'upstream/', '../upstream_1.0.orig.tar', 'upstream')
-    >>> gbp.command_wrappers.Command('gzip', [ '-n', '%s/../upstream_1.0.orig.tar' % repo_dir])()
+    >>> gbp.command_wrappers.Command('gzip', [ '-n', '%s/../upstream_1.0.orig.tar' % dirs['repo']])()
     """
 
+
 def test_pristine_tar_commit():
     """
     Commit the delta to the pristine-tar branch
@@ -84,10 +97,11 @@ def test_pristine_tar_commit():
          - L{gbp.deb.pristinetar.DebianPristineTar.commit}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.pristine_tar.commit('../upstream_1.0.orig.tar.gz', 'upstream')
     """
 
+
 def test_pristine_has_commit():
     """
     Find delta on the pristine tar branch
@@ -97,7 +111,7 @@ def test_pristine_has_commit():
          - L{gbp.pkg.pristinetar.PristineTar.get_commit}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.pristine_tar.has_commit('upstream', '1.0', 'bzip2')
     False
     >>> repo.pristine_tar.has_commit('upstream', '1.0', 'gzip')
@@ -110,6 +124,7 @@ def test_pristine_has_commit():
     True
     """
 
+
 def test_pristine_tar_checkout():
     """
     Checkout a tarball using pristine-tar
@@ -118,10 +133,11 @@ def test_pristine_tar_checkout():
          - L{gbp.deb.pristinetar.DebianPristineTar.checkout}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.pristine_tar.checkout('upstream', '1.0', 'gzip', '..')
     """
 
+
 def test_pristine_tar_checkout_nonexistent():
     """
     Checkout a tarball that does not exist using pristine-tar
@@ -130,7 +146,7 @@ def test_pristine_tar_checkout_nonexistent():
          - L{gbp.deb.pristinetar.DebianPristineTar.checkout}
 
     >>> import gbp.deb.git
-    >>> repo = gbp.deb.git.DebianGitRepository(repo_dir)
+    >>> repo = gbp.deb.git.DebianGitRepository(dirs['repo'])
     >>> repo.pristine_tar.checkout('upstream', '1.1', 'gzip', '..')
     Traceback (most recent call last):
     ...
@@ -138,11 +154,5 @@ def test_pristine_tar_checkout_nonexistent():
     pristine-tar: git show refs/heads/pristine-tar:upstream_1.1.orig.tar.gz.delta failed
     """
 
-def test_teardown():
-    """
-    Perform the teardown
-
-    >>> context.teardown()
-    """
 
 # vim:et:ts=4:sw=4:et:sts=4:ai:set list listchars=tab\:»·,trail\:·:
